<?php

namespace Dobley;

class AutoLoader
{
	public function __construct()
	{
		 spl_autoload_register(array($this, 'classLoader'));
	}

	/**
	 * Class loader
	 *
	 * @access public
	 * @param string $class
	 * @return void
	 */
	public function classLoader($class)
	{
		// We need to get the namecpace
		if($namespaces = self::getNamespaces($class)) {
			// Class name should be last element of the array
			$className = array_pop($namespaces);
		} else {
			$className = $class;
		}
		// If the class is already loaded there is no need to load it again
		if(class_exists($className, false)) return true;

		if(is_readable(LIB_PATH . "/" . $className . '.php')) {
			require_once LIB_PATH . "/" . $className . '.php';
		}
	}

	/**
	 * Get Namespaces
	 *
	 * @access public
	 * @param string $class namespaced class name
	 * @return mixed list of namespace levels or NULL if none found
	 */
	public static function getNamespaces($class)
	{
		if (self::hasNamespace($class)) {
			return explode('\\', $class);
		}
		return NULL;
	}

	/**
	 * Determines if a class name has a namespace element
	 *
	 * @access public
	 * @param string $class
	 * @return bool
	 */
	public static function hasNamespace($class)
	{
		if (strpos($class, '\\') !== false) {
			return true;
		}
		return false;
	}
}